@@ -19,6 +19,8 @@
  */
 package com.eteks.sweethome3d.j3d;
 
+import java.awt.GraphicsDevice;
+import java.awt.GraphicsEnvironment;
 import java.awt.Shape;
 import java.awt.geom.Area;
 import java.awt.geom.PathIterator;
@@ -31,14 +33,17 @@ import java.util.List;
 import java.util.Map;
 import java.util.WeakHashMap;
 
+import javax.media.j3d.Appearance;
 import javax.media.j3d.BranchGroup;
 import javax.media.j3d.ColoringAttributes;
 import javax.media.j3d.LineAttributes;
 import javax.media.j3d.Material;
 import javax.media.j3d.PolygonAttributes;
+import javax.media.j3d.RenderingAttributes;
 import javax.media.j3d.Texture;
 import javax.media.j3d.TextureAttributes;
 import javax.media.j3d.Transform3D;
+import javax.media.j3d.TransparencyAttributes;
 import javax.vecmath.Color3f;
 import javax.vecmath.Vector3d;
 import javax.vecmath.Vector3f;
@@ -46,18 +51,50 @@ import javax.vecmath.Vector3f;
 import com.eteks.sweethome3d.model.Home;
 import com.eteks.sweethome3d.model.HomeTexture;
 import com.eteks.sweethome3d.model.Room;
+import com.eteks.sweethome3d.model.UserPreferences;
+import com.eteks.sweethome3d.tools.OperatingSystem;
 
 /**
  * Root of a branch that matches a home object.
  */
 public abstract class Object3DBranch extends BranchGroup {
-  // The coloring attributes used for drawing outline
+  private static float screenScaleFactor = 1;
+
+  static {
+    try {
+      if (OperatingSystem.isMacOSX()
+          && OperatingSystem.isJavaVersionGreaterOrEqual("1.9")) {
+        // Use a thicker line width for Retina screens
+        GraphicsDevice screenDevice = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice();
+        Number scaleFactor = (Number)screenDevice.getClass().getDeclaredMethod("getScaleFactor").invoke(screenDevice);
+        if (scaleFactor instanceof Number && scaleFactor.floatValue() > 1f) {
+          screenScaleFactor = scaleFactor.floatValue();
+        }
+      }
+    } catch (Exception ex) {
+      // Ignore environments without getScaleFactor
+    }
+  }
+
+  protected static final float LINE_WIDTH_SCALE_FACTOR = screenScaleFactor;
+
+  // The attributes used for drawing outline
   protected static final ColoringAttributes OUTLINE_COLORING_ATTRIBUTES =
       new ColoringAttributes(new Color3f(0.16f, 0.16f, 0.16f), ColoringAttributes.FASTEST);
   protected static final PolygonAttributes OUTLINE_POLYGON_ATTRIBUTES =
       new PolygonAttributes(PolygonAttributes.POLYGON_LINE, PolygonAttributes.CULL_NONE, 0);
   protected static final LineAttributes OUTLINE_LINE_ATTRIBUTES =
-      new LineAttributes(0.5f, LineAttributes.PATTERN_SOLID, true);
+      new LineAttributes(0.5f * LINE_WIDTH_SCALE_FACTOR, LineAttributes.PATTERN_SOLID, true);
+
+  // The attributes used for drawing selection
+  protected static final ColoringAttributes SELECTION_COLORING_ATTRIBUTES =
+      new ColoringAttributes(new Color3f(0, 0, 0.7102f), ColoringAttributes.SHADE_FLAT);
+  protected static final PolygonAttributes  SELECTION_POLYGON_ATTRIBUTES =
+      new PolygonAttributes(PolygonAttributes.POLYGON_LINE, PolygonAttributes.CULL_NONE, 0);
+  protected static final LineAttributes     SELECTION_LINE_ATTRIBUTES =
+      new LineAttributes(LINE_WIDTH_SCALE_FACTOR * 3.5f, LineAttributes.PATTERN_SOLID, true);
+  protected static final TransparencyAttributes SELECTION_TRANSPARENCY_ATTRIBUTES =
+      new TransparencyAttributes(TransparencyAttributes.NICEST, 0.6f);
 
   protected static final Integer  DEFAULT_COLOR         = 0xFFFFFF;
   protected static final Integer  DEFAULT_AMBIENT_COLOR = 0x333333;
@@ -65,7 +102,7 @@ public abstract class Object3DBranch extends BranchGroup {
 
   private static final Map<Long, Material>                materials = new HashMap<Long, Material>();
   private static final Map<TextureKey, TextureAttributes> textureAttributes = new HashMap<TextureKey, TextureAttributes>();
-  private static final Map<Home, Map<Texture, Texture>>   homesTextures = new WeakHashMap<Home, Map<Texture, Texture>>();
+  private static final Map<Object, Map<Texture, Texture>> contextTextures = new WeakHashMap<Object, Map<Texture, Texture>>();
 
   static {
     DEFAULT_MATERIAL.setCapability(Material.ALLOW_COMPONENT_READ);
@@ -73,6 +110,44 @@ public abstract class Object3DBranch extends BranchGroup {
     DEFAULT_MATERIAL.setSpecularColor(0, 0, 0);
   }
 
+  private final Home home;
+  private final UserPreferences preferences;
+  private final Object context;
+
+  public Object3DBranch() {
+    this.home = null;
+    this.preferences = null;
+    this.context = null;
+  }
+
+  public Object3DBranch(Object item, Home home, UserPreferences preferences, Object context) {
+    this.context = context;
+    setUserData(item);
+    this.home = home;
+    this.preferences = preferences;
+  }
+
+  /**
+   * Returns home instance or <code>null</code>.
+   */
+  public Home getHome() {
+    return this.home;
+  }
+
+  /**
+   * Returns user preferences.
+   */
+  public UserPreferences getUserPreferences() {
+    return this.preferences;
+  }
+
+  /**
+   * Returns the context in which this object is used.
+   */
+  public Object getContext() {
+    return this.context;
+  }
+
   /**
    * Updates this branch from the home object.
    */
@@ -83,20 +158,32 @@ public abstract class Object3DBranch extends BranchGroup {
    * the texture itself if <code>home</code> is <code>null</code>.
    * As sharing textures across universes might cause some problems,
    * it's safer to handle a copy of textures for a given home.
+   * @deprecated Use {@link #getContextTexture(Texture, Object)} which context
+   *    parameter may be equal to different contexts for a given home
    */
   protected Texture getHomeTextureClone(Texture texture, Home home) {
-    if (home == null || texture == null) {
+    return getContextTexture(texture, home);
+  }
+
+  /**
+   * Returns a cloned instance of texture shared per <code>context</code> or
+   * the texture itself if <code>context</code> is <code>null</code>.
+   * As sharing textures across universes might cause some problems,
+   * it's safer to handle a copy of textures for a given context.
+   */
+  protected Texture getContextTexture(Texture texture, Object context) {
+    if (context == null || texture == null) {
       return texture;
     } else {
-      Map<Texture, Texture> homeTextures = homesTextures.get(home);
-      if (homeTextures == null) {
-        homeTextures = new WeakHashMap<Texture, Texture>();
-        homesTextures.put(home, homeTextures);
+      Map<Texture, Texture> contextTextures = Object3DBranch.contextTextures.get(context);
+      if (contextTextures == null) {
+        contextTextures = new WeakHashMap<Texture, Texture>();
+        Object3DBranch.contextTextures.put(context, contextTextures);
       }
-      Texture clonedTexture = homeTextures.get(texture);
+      Texture clonedTexture = contextTextures.get(texture);
       if (clonedTexture == null) {
         clonedTexture = (Texture)texture.cloneNodeComponent(false);
-        homeTextures.put(texture, clonedTexture);
+        contextTextures.put(texture, clonedTexture);
       }
       return clonedTexture;
     }
@@ -260,6 +347,22 @@ public abstract class Object3DBranch extends BranchGroup {
     }
   }
 
+  /**
+   * Returns an appearance for selection shapes.
+   */
+  protected Appearance getSelectionAppearance() {
+    Appearance selectionAppearance = new Appearance();
+    selectionAppearance.setColoringAttributes(SELECTION_COLORING_ATTRIBUTES);
+    selectionAppearance.setPolygonAttributes(SELECTION_POLYGON_ATTRIBUTES);
+    selectionAppearance.setLineAttributes(SELECTION_LINE_ATTRIBUTES);
+    selectionAppearance.setTransparencyAttributes(SELECTION_TRANSPARENCY_ATTRIBUTES);
+    RenderingAttributes renderingAttributes = new RenderingAttributes();
+    renderingAttributes.setCapability(RenderingAttributes.ALLOW_VISIBLE_WRITE);
+    selectionAppearance.setRenderingAttributes(renderingAttributes);
+    selectionAppearance.setCapability(Appearance.ALLOW_RENDERING_ATTRIBUTES_READ);
+    return selectionAppearance;
+  }
+
   /**
    * Returns the list of polygons points matching the given <code>area</code>.
    */
