@@ -40,6 +40,7 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
   private final String              name;
   private final String              description;
   private final String              information;
+  private final String              license;
   private final String []           tags;
   private final Long                creationDate;
   private final Float               grade;
@@ -71,10 +72,10 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
   private final BigDecimal          price;
   private final BigDecimal          valueAddedTaxPercentage;
   private final String              currency;
-  private final Map<String, String> properties;
+  private final Map<String, Object> properties;
 
-  private FurnitureCategory         category;
-  private byte []                   filterCollationKey;
+  private FurnitureCategory          category;
+  private byte []                    filterCollationKey;
 
   private static final Collator               COMPARATOR;
   private static final Map<String, byte [][]> recentFilters;
@@ -309,9 +310,9 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
                                  float [][] modelRotation, String creator,
                                  boolean resizable, boolean deformable, boolean texturable,
                                  BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency) {
-    this(id, name, description, information, tags, creationDate, grade, icon, planIcon, model, width, depth,
+    this(id, name, description, null, information, tags, creationDate, grade, icon, planIcon, model, width, depth,
         height, elevation, 1f, movable, false, staircaseCutOutShape, null, modelRotation, 0, null, creator, resizable, deformable,
-        texturable, true, price, valueAddedTaxPercentage, currency, null, (float)Math.PI / 8, 0, 1, true, false);
+        texturable, true, price, valueAddedTaxPercentage, currency, null, null, (float)Math.PI / 8, 0, 1, true, false);
   }
 
   /**
@@ -562,10 +563,68 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
                                  boolean resizable, boolean deformable, boolean texturable, boolean horizontallyRotatable,
                                  BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency,
                                  Map<String, String> properties) {
-    this(id, name, description, information, tags, creationDate, grade, icon, planIcon, model, width, depth,
-        height, elevation, dropOnTopElevation, movable, false, staircaseCutOutShape, null, modelRotation, modelFlags,
+    this(id, name, description, information, null, tags, creationDate, grade, icon, planIcon, model, width, depth,
+        height, elevation, dropOnTopElevation, movable, staircaseCutOutShape, modelRotation, modelFlags,
         modelSize, creator, resizable, deformable, texturable, horizontallyRotatable,
-        price, valueAddedTaxPercentage, currency, properties, (float)Math.PI / 8, 0, 1, true, false);
+        price, valueAddedTaxPercentage, currency, properties, null);
+  }
+
+  /**
+   * Creates an unmodifiable catalog piece of furniture of the default catalog.
+   * @param id    the id of the new piece or <code>null</code>
+   * @param name  the name of the new piece
+   * @param description the description of the new piece
+   * @param information additional information associated to the new piece
+   * @param license license of the new piece
+   * @param tags tags associated to the new piece
+   * @param creationDate creation date of the new piece in milliseconds since the epoch
+   * @param grade grade of the piece of furniture or <code>null</code>
+   * @param icon content of the icon of the new piece
+   * @param planIcon content of the icon of the new piece displayed in plan
+   * @param model content of the 3D model of the new piece
+   * @param width  the width in centimeters of the new piece
+   * @param depth  the depth in centimeters of the new piece
+   * @param height  the height in centimeters of the new piece
+   * @param elevation  the elevation in centimeters of the new piece
+   * @param dropOnTopElevation  a percentage of the height at which should be placed
+   *            an object dropped on the new piece
+   * @param movable if <code>true</code>, the new piece is movable
+   * @param staircaseCutOutShape the shape used to cut out upper levels when they intersect
+   *            with the piece like a staircase
+   * @param modelRotation the rotation 3 by 3 matrix applied to the piece model
+   * @param modelFlags flags which should be applied to piece model
+   * @param modelSize size of the 3D model of the new piece
+   * @param creator the creator of the model
+   * @param resizable if <code>true</code>, the size of the new piece may be edited
+   * @param deformable if <code>true</code>, the width, depth and height of the new piece may
+   *            change independently from each other
+   * @param texturable if <code>false</code> this piece should always keep the same color or texture
+   * @param horizontallyRotatable if <code>false</code> this piece
+   *            should not rotate around an horizontal axis
+   * @param price the price of the new piece or <code>null</code>
+   * @param valueAddedTaxPercentage the Value Added Tax percentage applied to the
+   *             price of the new piece or <code>null</code>
+   * @param currency the price currency, noted with ISO 4217 code, or <code>null</code>
+   * @param properties additional properties associating a key to a value or <code>null</code>
+   * @param contents   additional contents associating a key to a value or <code>null</code>
+   * @since 7.2
+   */
+  public CatalogPieceOfFurniture(String id, String name, String description,
+                                 String information, String license,
+                                 String [] tags, Long creationDate, Float grade,
+                                 Content icon, Content planIcon, Content model,
+                                 float width, float depth, float height,
+                                 float elevation, float dropOnTopElevation,
+                                 boolean movable, String staircaseCutOutShape,
+                                 float [][] modelRotation, int modelFlags, Long modelSize, String creator,
+                                 boolean resizable, boolean deformable, boolean texturable, boolean horizontallyRotatable,
+                                 BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency,
+                                 Map<String, String> properties, Map<String, Content> contents) {
+    this(id, name, description, information, license, tags, creationDate, grade, icon, planIcon, model, width, depth,
+        height, elevation, dropOnTopElevation,
+        movable, false, staircaseCutOutShape, null, modelRotation, modelFlags,
+        modelSize, creator, resizable, deformable, texturable, horizontallyRotatable,
+        price, valueAddedTaxPercentage, currency, properties, contents, (float)Math.PI / 8, 0, 1, true, false);
   }
 
   /**
@@ -677,10 +736,10 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
                                  boolean movable, String staircaseCutOutShape,
                                  Integer color, float [][] modelRotation, boolean backFaceShown, Long modelSize,
                                  String creator, float iconYaw, boolean proportional) {
-    this(null, name, null, null, new String [0], System.currentTimeMillis(), null, icon, null, model, width, depth, height, elevation, 1f,
+    this(null, name, null, null, null, new String [0], System.currentTimeMillis(), null, icon, null, model, width, depth, height, elevation, 1f,
         movable, false, staircaseCutOutShape, color, modelRotation, backFaceShown ? SHOW_BACK_FACE : 0,
         modelSize, creator, true, true, true, true, null, null, null,
-        null, iconYaw, (float)(-Math.PI / 16), 1, proportional, true);
+        null, null, iconYaw, (float)(-Math.PI / 16), 1, proportional, true);
   }
 
   /**
@@ -711,13 +770,14 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
                                  boolean movable, String staircaseCutOutShape,
                                  Integer color, float [][] modelRotation, int modelFlags, Long modelSize,
                                  String creator, float iconYaw, float iconPitch, float iconScale, boolean proportional) {
-    this(null, name, null, null, new String [0], System.currentTimeMillis(), null, icon, null, model, width, depth, height, elevation, 1f,
+    this(null, name, null, null, null, new String [0], System.currentTimeMillis(), null, icon, null, model, width, depth, height, elevation, 1f,
         movable, false, staircaseCutOutShape, color, modelRotation, modelFlags, modelSize, creator, true, true, true, true, null, null, null,
-        null, iconYaw, iconPitch, iconScale, proportional, true);
+        null, null, iconYaw, iconPitch, iconScale, proportional, true);
   }
 
   private CatalogPieceOfFurniture(String id, String name, String description,
-                                  String information, String [] tags, Long creationDate, Float grade,
+                                  String information, String license,
+                                  String [] tags, Long creationDate, Float grade,
                                   Content icon, Content planIcon, Content model,
                                   float width, float depth, float height,
                                   float elevation, float dropOnTopElevation,
@@ -726,13 +786,14 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
                                   Long modelSize, String creator, boolean resizable,
                                   boolean deformable, boolean texturable, boolean horizontallyRotatable,
                                   BigDecimal price, BigDecimal valueAddedTaxPercentage, String currency,
-                                  Map<String, String> properties,
+                                  Map<String, String> properties, Map<String, Content> contents,
                                   float iconYaw, float iconPitch, float iconScale, boolean proportional,
                                   boolean modifiable) {
     this.id = id;
     this.name = name;
     this.description = description;
     this.information = information;
+    this.license = license;
     this.tags = tags;
     this.creationDate = creationDate;
     this.grade = grade;
@@ -753,11 +814,23 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
     this.price = price;
     this.valueAddedTaxPercentage = valueAddedTaxPercentage;
     this.currency = currency;
-    this.properties = properties == null || properties.size() == 0
-        ? Collections.<String, String>emptyMap()
-        : (properties.size() == 1
-            ? Collections.singletonMap(properties.keySet().iterator().next(), properties.values().iterator().next())
-            : new HashMap<String, String>(properties));
+    // Merge properties and contents in the same map
+    if (properties == null || properties.size() == 0) {
+      if (contents == null || contents.size() == 0) {
+        this.properties = Collections.<String, Object>emptyMap();
+      } else if (contents.size() == 1) {
+        this.properties = Collections.singletonMap(contents.keySet().iterator().next(), (Object)contents.values().iterator().next());
+      } else {
+        this.properties = new HashMap<String, Object>(contents);
+      }
+    } else if (properties.size() == 1 && (contents == null || contents.size() == 0)) {
+      this.properties = Collections.singletonMap(properties.keySet().iterator().next(), (Object)properties.values().iterator().next());
+    } else {
+      this.properties = new HashMap<String, Object>(properties);
+      if (contents != null) {
+        this.properties.putAll(contents);
+      }
+    }
     if (modelRotation == null) {
       this.modelRotation = IDENTITY_ROTATION;
     } else {
@@ -805,6 +878,14 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
     return this.information;
   }
 
+  /**
+   * Returns the license associated to this piece, or <code>null</code>.
+   * @since 7.2
+   */
+  public String getLicense() {
+    return this.license;
+  }
+
   /**
    * Returns the tags associated to this piece.
    * @since 3.6
@@ -933,12 +1014,14 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
   }
 
   /**
-   * Returns a deep copy of the given 3 by 3 matrix.
+   * Returns a deep copy of the given array.
    */
-  static float [][] deepClone(float [][] modelRotation) {
-    return new float [][] {{modelRotation [0][0], modelRotation [0][1], modelRotation [0][2]},
-                           {modelRotation [1][0], modelRotation [1][1], modelRotation [1][2]},
-                           {modelRotation [2][0], modelRotation [2][1], modelRotation [2][2]}};
+  static float [][] deepClone(float [][] array) {
+    float [][] clone = new float [array.length][];
+    for (int i = 0; i < array.length; i++) {
+      clone [i] = array [i].clone();
+    }
+    return clone;
   }
 
   /**
@@ -1073,10 +1156,15 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
 
   /**
    * Returns the value of an additional property <code>name</code> of this piece.
-   * @return the value of the property or <code>null</code> if it doesn't exist.
+   * @return the value of the property or <code>null</code> if it doesn't exist or if it's not a string.
    */
   public String getProperty(String name) {
-    return this.properties.get(name);
+    Object propertyValue = this.properties.get(name);
+    if (propertyValue instanceof String) {
+      return (String)propertyValue;
+    } else {
+      return null;
+    }
   }
 
   /**
@@ -1087,6 +1175,28 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
     return this.properties.keySet();
   }
 
+  /**
+   * Returns the value of an additional content <code>name</code> associated to this piece.
+   * @return the value of the content or <code>null</code> if it doesn't exist or if it's not a content.
+   * @since 7.2
+   */
+  public Content getContentProperty(String name) {
+    Object propertyValue = this.properties.get(name);
+    if (propertyValue instanceof Content) {
+      return (Content)propertyValue;
+    } else {
+      return null;
+    }
+  }
+
+  /**
+   * Returns <code>true</code> if the type of given additional property is a content.
+   * @since 7.2
+   */
+  public boolean isContentProperty(String name) {
+    return this.properties.get(name) instanceof Content;
+  }
+
   /**
    * Returns the category of this piece of furniture.
    */
@@ -1135,7 +1245,7 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
   /**
    * Returns <code>true</code> if this piece matches the given <code>filter</code> text.
    * Each substring of the <code>filter</code> is considered as a search criterion that can match
-   * the name, the category name, the creator, the description or the tags of this piece.
+   * the name, the category name, the creator, the license, the description or the tags of this piece.
    * @since 4.2
    */
   public boolean matchesFilter(String filter) {
@@ -1200,7 +1310,7 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
   }
 
   /**
-   * Returns the strings used as criteria for filtering (name, category, creator, description and tags).
+   * Returns the strings used as criteria for filtering (name, category, creator, license, description and tags).
    * @see CatalogPieceOfFurniture#matchesFilter(String)
    * @since 6.2
    */
@@ -1216,6 +1326,9 @@ public class CatalogPieceOfFurniture implements Comparable<CatalogPieceOfFurnitu
     if (getDescription() != null) {
       criteria.add(getDescription());
     }
+    if (getLicense() != null) {
+      criteria.add(getLicense());
+    }
     for (String tag : getTags()) {
       criteria.add(tag);
     }
