@@ -246,46 +246,26 @@ public class HomePrintableComponent extends JComponent implements Printable {
     }
 
     Rectangle clipBounds = g2D.getClipBounds();
+    FontMetrics fontMetrics = g2D.getFontMetrics(this.headerFooterFont);
     AffineTransform oldTransform = g2D.getTransform();
     final PlanView planView = this.controller.getPlanController().getView();
+    final List<Level> printedLevels = homePrint != null ? homePrint.getPrintedLevels() : null;
+    String levelName = "";
+    if (this.home.getSelectedLevel() != null) {
+      levelName = this.home.getSelectedLevel().getName();
+    }
+    String headerFormat = null;
+    String footerFormat = null;
     if (homePrint != null
         || fixedHeaderPageLabel != null
         || fixedFooterPageLabel != null) {
       if (homePrint != null) {
-        FontMetrics fontMetrics = g2D.getFontMetrics(this.headerFooterFont);
         float headerFooterHeight = fontMetrics.getAscent() + fontMetrics.getDescent() + HEADER_FOOTER_MARGIN;
 
-        // Retrieve variable values
-        int pageNumber = page + 1;
-        int pageCount = getPageCount();
-        String planScale = "?";
-        if (homePrint.getPlanScale() != null) {
-          planScale = "1/" + Math.round(1 / homePrint.getPlanScale());
-        } else {
-          float preferredScale = planView.getPrintPreferredScale(LengthUnit.inchToCentimeter((float)pageFormat.getImageableWidth() / 72),
-              LengthUnit.inchToCentimeter((float)pageFormat.getImageableHeight() / 72));
-          planScale = "1/" + Math.round(1 / preferredScale);
-        }
-        if (page == 0) {
-          this.printDate = new Date();
-        }
-        String homeName = this.home.getName();
-        if (homeName == null) {
-          homeName = "";
-        }
-        String levelName = "";
-        if (this.home.getSelectedLevel() != null) {
-          levelName = this.home.getSelectedLevel().getName();
-        }
-        String homePresentationName = this.controller.getContentManager().getPresentationName(
-             homeName, ContentManager.ContentType.SWEET_HOME_3D);
-        Object [] variableValues = new Object [] {
-            pageNumber, pageCount, planScale, this.printDate, homePresentationName, homeName, levelName};
-
         // Create header text
-        String headerFormat = homePrint.getHeaderFormat();
+        headerFormat = homePrint.getHeaderFormat();
         if (headerFormat != null) {
-          header = Variable.getMessageFormat(headerFormat).format(variableValues).trim();
+          header = formatHeaderFooter(headerFormat, pageFormat, homePrint, planView, page, levelName);
           if (header.length() > 0) {
             xHeader = ((float)pageFormat.getWidth() - fontMetrics.stringWidth(header)) / 2;
             yHeader = imageableY + fontMetrics.getAscent();
@@ -297,9 +277,9 @@ public class HomePrintableComponent extends JComponent implements Printable {
         }
 
         // Create footer text
-        String footerFormat = homePrint.getFooterFormat();
+        footerFormat = homePrint.getFooterFormat();
         if (footerFormat != null) {
-          footer = Variable.getMessageFormat(footerFormat).format(variableValues).trim();
+          footer = formatHeaderFooter(footerFormat, pageFormat, homePrint, planView, page, levelName);
           if (footer.length() > 0) {
             xFooter = ((float)pageFormat.getWidth() - fontMetrics.stringWidth(footer)) / 2;
             yFooter = imageableY + imageableHeight - fontMetrics.getDescent();
@@ -346,10 +326,9 @@ public class HomePrintableComponent extends JComponent implements Printable {
           && (homePrint == null
               || homePrint.isPlanPrinted()
               || homePrint.isView3DPrinted())) {
-        final Level selectedLevel = home.getSelectedLevel();
+        final Level selectedLevel = this.home.getSelectedLevel();
         filteredFurnitureView = (FurnitureView)furnitureView;
         furnitureFilter = filteredFurnitureView.getFurnitureFilter();
-        final List<Level> printedLevels = homePrint != null ? homePrint.getPrintedLevels() : null;
         filteredFurnitureView.setFurnitureFilter(new FurnitureView.FurnitureFilter() {
             public boolean include(Home home, HomePieceOfFurniture piece) {
               // Print only furniture at selected level when the plan or the 3D view is printed
@@ -367,10 +346,30 @@ public class HomePrintableComponent extends JComponent implements Printable {
         // Restore previous filter
         filteredFurnitureView.setFurnitureFilter(furnitureFilter);
       }
-      if (pageExists == PAGE_EXISTS
-          && !this.printablePages.contains(page)) {
-        this.printablePages.add(page);
-        this.furniturePageCount++;
+      if (pageExists == PAGE_EXISTS) {
+        if (!this.printablePages.contains(page)) {
+          this.printablePages.add(page);
+          this.furniturePageCount++;
+        }
+
+        if (printedLevels != null) {
+          // Adjust header and footer texts
+          levelName = printedLevels.size() == 1
+              ? printedLevels.get(0).getName()
+              : "";
+          if (headerFormat != null) {
+            header = formatHeaderFooter(headerFormat, pageFormat, homePrint, planView, page, levelName);
+            if (header.length() > 0) {
+              xHeader = ((float)pageFormat.getWidth() - fontMetrics.stringWidth(header)) / 2;
+            }
+          }
+          if (footerFormat != null) {
+            footer = formatHeaderFooter(footerFormat, pageFormat, homePrint, planView, page, levelName);
+            if (footer.length() > 0) {
+              xFooter = ((float)pageFormat.getWidth() - fontMetrics.stringWidth(footer)) / 2;
+            }
+          }
+        }
       }
     }
     if (pageExists == NO_SUCH_PAGE
@@ -378,10 +377,32 @@ public class HomePrintableComponent extends JComponent implements Printable {
         && (homePrint == null || homePrint.isPlanPrinted())) {
       // Try to print next plan view page
       pageExists = ((Printable)planView).print(g2D, pageFormat, page - this.furniturePageCount);
-      if (pageExists == PAGE_EXISTS
-          && !this.printablePages.contains(page)) {
-        this.printablePages.add(page);
-        this.planPageCount++;
+      if (pageExists == PAGE_EXISTS) {
+        if (!this.printablePages.contains(page)) {
+          this.printablePages.add(page);
+          this.planPageCount++;
+        }
+
+        if (printedLevels != null) {
+          // Adjust header and footer texts
+          levelName = this.home.getPrint().getPlanScale() == null
+              ? printedLevels.get(page - this.furniturePageCount).getName()
+              : printedLevels.size() == 1
+                  ? printedLevels.get(0).getName()
+                  : "";
+          if (headerFormat != null) {
+            header = formatHeaderFooter(headerFormat, pageFormat, homePrint, planView, page, levelName);
+            if (header.length() > 0) {
+              xHeader = ((float)pageFormat.getWidth() - fontMetrics.stringWidth(header)) / 2;
+            }
+          }
+          if (footerFormat != null) {
+            footer = formatHeaderFooter(footerFormat, pageFormat, homePrint, planView, page, levelName);
+            if (footer.length() > 0) {
+              xFooter = ((float)pageFormat.getWidth() - fontMetrics.stringWidth(footer)) / 2;
+            }
+          }
+        }
       }
     }
     View view3D = this.controller.getHomeController3D().getView();
@@ -455,6 +476,37 @@ public class HomePrintableComponent extends JComponent implements Printable {
     }
   }
 
+  /**
+   * Returns the header or footer text from the given format.
+   */
+  private String formatHeaderFooter(String headerFooterFormat,
+                                    PageFormat pageFormat, HomePrint homePrint, PlanView planView,
+                                    int page, String levelName) {
+    // Retrieve variable values
+    int pageNumber = page + 1;
+    int pageCount = getPageCount();
+    String planScale = "?";
+    if (homePrint.getPlanScale() != null) {
+      planScale = "1/" + Math.round(1 / homePrint.getPlanScale());
+    } else {
+      float preferredScale = planView.getPrintPreferredScale(LengthUnit.inchToCentimeter((float)pageFormat.getImageableWidth() / 72),
+          LengthUnit.inchToCentimeter((float)pageFormat.getImageableHeight() / 72));
+      planScale = "1/" + Math.round(1 / preferredScale);
+    }
+    if (page == 0) {
+      this.printDate = new Date();
+    }
+    String homeName = this.home.getName();
+    if (homeName == null) {
+      homeName = "";
+    }
+    String homePresentationName = this.controller.getContentManager().getPresentationName(
+         homeName, ContentManager.ContentType.SWEET_HOME_3D);
+    Object [] variableValues = new Object [] {
+        pageNumber, pageCount, planScale, this.printDate, homePresentationName, homeName, levelName};
+    return Variable.getMessageFormat(headerFooterFormat).format(variableValues).trim();
+  }
+
   /**
    * Sets the page currently painted by this component.
    */
